@@ -42,6 +42,8 @@ setClass("Cov", representation(call = "language",
                               "VIRTUAL"),
                  prototype=list(det=-1))
 
+setClassUnion("UCov", c("Cov", "NULL"))
+
 setClass("SummaryCov", representation(covobj = "Cov",
                               evals = "vector"))
 
@@ -315,7 +317,9 @@ setClass("PcaRobust", representation("VIRTUAL"),
                     contains="Pca")
 
 setClass("PcaHubert", representation(alpha = "numeric",
-                                  quan = "numeric"),
+                                     quan = "numeric",
+                                     skew = "logical",
+                                     ao = "Uvector"),
                     contains="PcaRobust")
 setClass("PcaLocantore", representation(),
                     contains="PcaRobust")
@@ -337,6 +341,8 @@ setClass("Lda", representation(call = "language",
                                method = "character",
                                X = "Umatrix",
                                grp = "factor",
+                               covobj  = "UCov",
+                               control  = "UCovControl",
                                "VIRTUAL"))
 
 setClass("SummaryLda", representation(ldaobj = "Lda"))
