@@ -1,3 +1,61 @@
+######
+##  VT::14.01.2020
+##
+##
+##  roxygen2::roxygenise("C:/projects/statproj/R/rrcov")
+##
+#' Calculates the points for drawing a confidence ellipsoid
+#'
+#' @description A simple function to calculate the points of
+#'   a confidence ellipsoid, by default \code{dist=qchisq(0.975, 2)}
+#' @param loc location vector
+#' @param cov a \code{pXp} covariance matrix
+#' @param crit the confidence level, default is \code{crit=0.975}
+#' @return  A matrix with two columns containing the calculated points.
+#'
+#' @examples
+#'
+#' data(hbk)
+#' cc <- cov.wt(hbk)
+#' e1 <- getEllipse(loc=cc$center[1:2], cov=cc$cov[1:2,1:2])
+#' e2 <- getEllipse(loc=cc$center[1:2], cov=cc$cov[1:2,1:2], crit=0.99)
+#' plot(X2~X1, data=hbk,
+#'     xlim=c(min(X1, e1[,1], e2[,1]), max(X1,e1[,1], e2[,1])),
+#'     ylim=c(min(X2, e1[,2], e2[,2]), max(X2,e1[,2], e2[,2])))
+#' lines(e1, type="l", lty=1, col="red")
+#' lines(e2, type="l", lty=2, col="blue")
+#' legend("topleft", legend=c(0.975, 0.99), lty=1:2, col=c("red", "blue"))
+#'
+#' @export
+#' @author Valentin Todorov, \email{valentin.todorov@@chello.at}
+#'
+getEllipse <- function(loc=c(0, 0), cov=matrix(c(1,0,0,1), ncol=2), crit=0.975) {
+    ## A simple function to calculate the points of a confidence ellipsoid,
+    ##  by default \code{dist=qchisq(0.975, 2)}
+    ## input: data set location and covariance estimate, cutoff
+
+    if (length(d <- dim(cov)) != 2 || (p <- d[1]) != d[2])
+        stop("'cov' must be p x p  cov-matrix defining an ellipsoid")
+
+    dist <- sqrt(qchisq(crit, p))
+    A <- solve(cov)
+    eA <- eigen(A)
+    ev <- eA$values
+
+    lambda1 <- max(ev)
+    lambda2 <- min(ev)
+    eigvect <- eA$vectors[, order(ev)[2]]
+
+    z <- seq(0, 2 * pi, by = 0.01)
+    z1 <- dist/sqrt(lambda1) * cos(z)
+    z2 <- dist/sqrt(lambda2) * sin(z)
+    alfa <- atan(eigvect[2]/eigvect[1])
+    r <- matrix(c(cos(alfa),  - sin(alfa), sin(alfa), cos(alfa)), ncol = 2)
+    z <- t(t(cbind(z1, z2) %*% r) + loc)    #   xmin <- min(x, z[, 1])
+
+    z
+}
+
 ## Internal class defining a default legend parameters to be used
 ##  in plots with both robust and classical lines in the same panel
 ##  (e.g. scree plot, tolerance ellipses)
@@ -11,7 +69,7 @@ setClass(".Legend", representation( leg = "logical",        # whether to draw a
                                      pos = "topright",
                                      txt = c("robust", "classical"),
                                      col = c("red", "blue"),
-                                     lty = c("solid", "dotted"),
+                                     lty = c("solid", "solid"),
                                      pch = c(1,24)))
 
 
@@ -111,7 +169,7 @@ myscreeplot <- function(rcov, ccov) {
 
 .qqplot <- function(x, p, cutoff, classic=FALSE, id.n,
     main=eval(substitute(expression(paste(chi^2, " QQ-Plot")))),
-    xlab=eval(substitute(expression(paste("Sqrt of the quantiles of the ", chi^2, " distribution")))),
+    xlab=eval(substitute(expression(paste("Sqrt of the quantiles of the ", chi[p]^2, " distribution")), list(p=p))),
     ylab=paste(if(classic) "Mahalanobis" else "Robust", "distance"),
     labs=1:length(x),
     ...)
@@ -169,32 +227,10 @@ myscreeplot <- function(rcov, ccov) {
     main = "Tolerance ellipse (97.5%)",
     xlab = "",
     ylab = "",
-    labs,
+    labs, legend.position="bottomright",
     ...)
 {
 
-## MM: This is nothing else but a version  cluster::ellipsoidPoints() !! -- FIXME
-    ellips <- function(loc, cov) {
-        ## calculates a 97,5% ellipsoid
-        ## input: data set, location and covariance estimate, cutoff
-
-        dist <- sqrt(qchisq(0.975, 2))
-        A <- solve(cov)
-        eA <- eigen(A)
-        ev <- eA$values
-        lambda1 <- max(ev)
-        lambda2 <- min(ev)
-        eigvect <- eA$vectors[, order(ev)[2]]
-        z <- seq(0, 2 * pi, by = 0.01)
-        z1 <- dist/sqrt(lambda1) * cos(z)
-        z2 <- dist/sqrt(lambda2) * sin(z)
-        alfa <- atan(eigvect[2]/eigvect[1])
-        r <- matrix(c(cos(alfa),  - sin(alfa), sin(alfa), cos(alfa)), ncol = 2)
-        z <- t(t(cbind(z1, z2) %*% r) + loc)    #   xmin <- min(x, z[, 1])
-
-        z
-    }
-
     leg <- new(".Legend")
 
     if(missing(rcov) || is.null(rcov)){
@@ -216,7 +252,7 @@ myscreeplot <- function(rcov, ccov) {
     r.loc <- getCenter(rcov)
     if(length(r.loc) == 0 ||  length(r.cov) == 0)
         stop("Invalid 'rcov' object: attributes center and cov missing!")
-    z1 <- ellips(loc = r.loc, cov = r.cov)
+    z1 <- getEllipse(loc = r.loc, cov = r.cov)
     rd <- sqrt(getDistance(rcov))
     x1 <- c(min(data[, 1], z1[, 1]), max(data[,1],z1[,1]))
     y1 <- c(min(data[, 2], z1[, 2]), max(data[,2],z1[,2]))
@@ -228,7 +264,7 @@ myscreeplot <- function(rcov, ccov) {
         if(length(c.loc) == 0 ||  length(c.cov) == 0)
             stop("Invalid 'ccov' object: attributes center and cov missing!")
         classic <- TRUE
-        z2 <- ellips(loc = c.loc, cov = c.cov)
+        z2 <- getEllipse(loc=c.loc, cov=c.cov)
         md <- sqrt(getDistance(ccov))
         x1 <- c(min(data[, 1], z1[, 1], z2[, 1]), max(data[,1],z1[,1], z2[,1]))
         y1 <- c(min(data[, 2], z1[, 2], z2[, 2]), max(data[,2],z1[,2], z2[,2]))
@@ -267,7 +303,7 @@ myscreeplot <- function(rcov, ccov) {
     if(classic){
         points(z2, type = "l", lty=leg@lty[2], col=leg@col[2])
         if(leg@leg)
-            legend("bottomright", leg@txt, pch=leg@pch, lty=leg@lty, col=leg@col)
+            legend(legend.position, leg@txt, pch=leg@pch, lty=leg@lty, col=leg@col)
     }
 
     invisible()
@@ -291,24 +327,8 @@ myscreeplot <- function(rcov, ccov) {
     txt.leg=c("robust", "classical"), col.leg = c("red", "blue"),
     lty.leg=c("solid", "dashed"),
     xlim,
-    ylim)
+    ylim, ...)
 {
-    ellips <- function(loc, cov) {
-        dist <- sqrt(qchisq(0.975, 2))
-        A <- solve(cov)
-        eA <- eigen(A)
-        ev <- eA$values
-        lambda1 <- max(ev)
-        lambda2 <- min(ev)
-        eigvect <- eA$vectors[, order(ev)[2]]
-        z <- seq(0, 2 * pi, by = 0.01)
-        z1 <- dist/sqrt(lambda1) * cos(z)
-        z2 <- dist/sqrt(lambda2) * sin(z)
-        alfa <- atan(eigvect[2]/eigvect[1])
-        r <- matrix(c(cos(alfa), -sin(alfa), sin(alfa), cos(alfa)),
-            ncol = 2)
-        t(loc + t(cbind(z1, z2) %*% r))
-    }
     if (is.data.frame(x))
         x <- data.matrix(x)
     if (!is.matrix(x) || !is.numeric(x))
@@ -334,8 +354,8 @@ myscreeplot <- function(rcov, ccov) {
     }
 
     xM <- colMeans(x)
-    z1 <- ellips(loc = xM, cov = n/(n - 1) * cov.wt(x)$cov)
-    z2 <- ellips(loc = x.loc, cov = x.cov)
+    z1 <- getEllipse(loc=xM, cov=n/(n - 1) * cov.wt(x)$cov)
+    z2 <- getEllipse(loc=x.loc, cov=x.cov)
     ## VT::09.06.200
     ## if no need of the classic ellipse - set it to the robust one
     ## otherwise the xlim and ylim will be set to fit both ellipses
@@ -361,7 +381,7 @@ myscreeplot <- function(rcov, ccov) {
     if(missing(ylim))
         ylim <- y1
 
-    plot(x, xlim = xlim, ylim = ylim, xlab = xlab, ylab = ylab, main = main, sub=sub)
+    plot(x, xlim = xlim, ylim = ylim, xlab = xlab, ylab = ylab, main = main, sub=sub, ...)
     box()
     if(id.n > 0) {
         xrange <- par("usr")
@@ -369,10 +389,10 @@ myscreeplot <- function(rcov, ccov) {
         text(x[ind, 1] + xrange/50, x[ind, 2], ind)
     }
 
-    points(z2, type = "l", lty = lty.leg[1], col = col.leg[1])
+    points(z2, type = "l", lty = lty.leg[1], col = col.leg[1], ...)
     if (classic) {
-        points(z1, type = "l", lty = lty.leg[2], col = col.leg[2])
-        legend("bottomright", txt.leg, lty = lty.leg, col = col.leg)
+        points(z1, type = "l", lty = lty.leg[2], col = col.leg[2], ...)
+        legend("bottomright", txt.leg, lty = lty.leg, col = col.leg, ...)
     }
     invisible()
 }
@@ -476,7 +496,6 @@ myscreeplot <- function(rcov, ccov) {
         X <- cbind(x,y)
         C.ls <- cov(X)
         m.ls <- colMeans(X)
-        d2.99 <- qchisq(0.975, df = 2)
 
         ix <- which.ij(x, y, getData(obj))
         ## cat("\npanel.ellipse: ", ix$i, ix$j,"\n")
@@ -484,8 +503,8 @@ myscreeplot <- function(rcov, ccov) {
         C.rr <- getCov(obj)[c(ix$i,ix$j), c(ix$i,ix$j)]
         m.rr <- getCenter(obj)[c(ix$i,ix$j)]
 
-        e.class <- ellipsoidPoints(C.ls, d2.99, loc=m.ls)
-        e.rob <- ellipsoidPoints(C.rr, d2=d2.99, loc=m.rr)
+        e.class <- getEllipse(loc=m.ls, cov=C.ls, crit=0.975)
+        e.rob <- getEllipse(loc=m.rr, cov=C.rr, crit=0.975)
 
         xmin <- min(c(min(x), min(e.class[,1]), min(e.rob[,1])))
         xmax <- max(c(max(x), max(e.class[,1]), max(e.rob[,1])))
@@ -593,7 +612,7 @@ myscreeplot <- function(rcov, ccov) {
 ##
 .xyqqchi2 <- function(obj, cutoff,
     main=eval(substitute(expression(paste(chi^2, " QQ-Plot")))),
-    xlab=eval(substitute(expression(paste("Sqrt of the quantiles of the ", chi^2, " distribution")))),
+    xlab=eval(substitute(expression(paste("Sqrt of the quantiles of the ", chi[p]^2, " distribution")), list(p=p))),
     ylab="Mahalanobis distance",
     col="darkred",
     labs,
